@@ -45,7 +45,7 @@
 //! For more google play usage examples, such as specifying a device configuration, timezone or
 //! locale, refer to the [`USAGE-google-play.md`](USAGE-google-play.md) document.
 //!
-//! Or, to download from the F-Droid open source repository:
+//! To download from the F-Droid open source repository:
 //!
 //! ```shell
 //! apkeep -a org.mozilla.fennec_fdroid -d f-droid .
@@ -54,6 +54,12 @@
 //! For more F-Droid usage examples, such as downloading from F-Droid mirrors or other F-Droid
 //! repositories, refer to the [`USAGE-fdroid.md`](USAGE-fdroid.md) document.
 //!
+//! Or, to download from the Huawei AppGallery:
+//!
+//! ```shell
+//! apkeep -a com.elysiumlabs.newsbytes -d huawei-app-gallery .
+//! ```
+//!
 //! To download a specific version of an APK (possible for APKPure or F-Droid), use the `@version`
 //! convention:
 //!
@@ -94,6 +100,7 @@
 //! * F-Droid (`-d f-droid`), a repository for free and open-source Android apps. `apkeep`
 //! verifies that these APKs are signed by the F-Droid maintainers, and alerts the user if an APK
 //! was downloaded but could not be verified
+//! * The Huawei AppGallery (`-d huawei-app-gallery`), an app store popular in China
 //!
 //! # Usage Note
 //!
@@ -111,17 +118,22 @@
 
 use std::collections::HashMap;
 use std::error::Error;
-use std::fs;
-use std::path::Path;
+use std::fs::{self, File};
+use std::io::{self, Write, Read};
+use std::path::{Path, PathBuf};
+
+use configparser::ini::Ini;
 
 mod cli;
 use cli::DownloadSource;
 
+mod config;
 mod consts;
 
 mod apkpure;
 mod fdroid;
 mod google_play;
+mod huawei_app_gallery;
 
 type CSVList = Vec<(String, Option<String>)>;
 fn fetch_csv_list(csv: &str, field: usize, version_field: Option<usize>) -> Result<CSVList, Box<dyn Error>> {
@@ -158,6 +170,23 @@ fn parse_csv_text(text: String, field: usize, version_field: Option<usize>) -> V
         .collect()
 }
 
+fn load_config(ini_file: Option<PathBuf>) -> Result<Ini, Box<dyn Error>> {
+    let mut conf = Ini::new();
+    let config_path = match ini_file {
+        Some(ini_file) => ini_file,
+        None => {
+            let mut config_path = config::config_dir()?;
+            config_path.push("apkeep.ini");
+            config_path
+        }
+    };
+    let mut config_fp = File::open(&config_path)?;
+    let mut contents = String::new();
+    config_fp.read_to_string(&mut contents)?;
+    conf.read(contents)?;
+    Ok(conf)
+}
+
 #[tokio::main]
 async fn main() {
     let usage = {
@@ -230,6 +259,9 @@ async fn main() {
             DownloadSource::FDroid => {
                 fdroid::list_versions(list, options).await;
             }
+            DownloadSource::HuaweiAppGallery => {
+                huawei_app_gallery::list_versions(list).await;
+            }
         }
     } else {
         let parallel: usize = matches.value_of_t("parallel").unwrap();
@@ -254,14 +286,50 @@ async fn main() {
                 apkpure::download_apps(list, parallel, sleep_duration, &outpath).await;
             }
             DownloadSource::GooglePlay => {
-                let username = matches.value_of("google_username").unwrap();
-                let password = matches.value_of("google_password").unwrap();
+                let mut username = matches.value_of("google_username").map(|v| v.to_string());
+                let mut password = matches.value_of("google_password").map(|v| v.to_string());
+
+                let ini_file = matches.value_of("ini").map(|ini_file| {
+                    match fs::canonicalize(ini_file) {
+                        Ok(ini_file) if Path::new(&ini_file).is_file() => {
+                            ini_file
+                        },
+                        _ => {
+                            println!("{}\n\nSpecified ini is not a valid file", usage);
+                            std::process::exit(1);
+                        },
+                    }
+                });
+
+                if username.is_none() || password.is_none() {
+                    if let Ok(conf) = load_config(ini_file) {
+                        if username.is_none() {
+                            username = conf.get("google", "username");
+                        }
+                        if password.is_none() {
+                            password = conf.get("google", "password");
+                        }
+                    }
+                }
+                
+                if username.is_none() {
+                    let mut prompt_username = String::new();
+                    print!("Username: ");
+                    io::stdout().flush().unwrap();
+                    io::stdin().read_line(&mut prompt_username).unwrap();
+                    username = Some(prompt_username);
+                }
+
+                if password.is_none() {
+                    password = Some(rpassword::prompt_password("Password: ").unwrap());
+                }
+
                 google_play::download_apps(
                     list,
                     parallel,
                     sleep_duration,
-                    username,
-                    password,
+                    &username.unwrap(),
+                    &password.unwrap(),
                     &outpath,
                     options,
                 )
@@ -275,6 +343,9 @@ async fn main() {
                     options,
                 ).await;
             }
+            DownloadSource::HuaweiAppGallery => {
+                huawei_app_gallery::download_apps(list, parallel, sleep_duration, &outpath).await;
+            }
         }
     }
 }
